/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.nec.connexive.thingpark.token.verifier;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.IOException;
import java.math.BigInteger;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

@WebServlet(name = "verifyToken", urlPatterns = {"/verifyToken"})
public class Verifier extends HttpServlet {

    protected static final ObjectMapper mapper = new ObjectMapper();
    private static final String QUERY_PARAMETER_HEADER = "X-Original-QP";
    private static final String BODY_HEADER = "X-Original-BODY";

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    @Produces(MediaType.APPLICATION_JSON)
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        StringBuilder sb = new StringBuilder();

        // The Application Server builds the <body-elements> as the concatenation, without separator.
        String dobyString = request.getHeader(BODY_HEADER);
        if (dobyString != null && !dobyString.isEmpty()) {
            JsonNode json = mapper.readTree(dobyString);
            if (json != null && json.get("DevEUI_uplink") != null) {
                json = json.get("DevEUI_uplink");
                String[] targetFields = {"CustomerID", "DevEUI", "FPort", "FCntUp", "payload_hex"};
                for (String targetField : targetFields) {
                    if (json.get(targetField) != null) {
                        sb.append(json.get(targetField).asText());
                    }
                }
            }
        }
        System.out.println("<body-elements>:" + sb.toString());

        // The Application Server retrieves the <query-parameters> WITHOUT the Token QP
        String queryString = URLDecoder.decode(request.getHeader(QUERY_PARAMETER_HEADER),"UTF-8");
        String queryParamToken = null;
        boolean isFirst = true;
        if (queryString != null && !queryString.isEmpty()) {
            String[] queryStringArray = queryString.split("&");
            for (String qs : queryStringArray) {
                if (qs.startsWith("Token=")) {
                    queryParamToken = qs.substring(6);
                } else if (isFirst) {
                    sb.append(qs);
                    isFirst = false;
                } else {
                    sb.append("&");
                    sb.append(qs);
                }
            }
        }
        System.out.println("<body-elements><query-parameters>:" + sb.toString());

        sb.append(ConfigReader.getLrcAsKey());
        System.out.println("<body-elements><query-parameters><LrcAsKey>:" + sb.toString());

        MessageDigest digest;
        String token = null;
        try {
            digest = MessageDigest.getInstance("SHA-256");
            digest.reset();
            digest.update(sb.toString().getBytes(StandardCharsets.UTF_8));
            token = String.format("%064x", new BigInteger(1, digest.digest()));
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(Verifier.class.getName()).log(Level.SEVERE, null, ex);
        }

        System.out.println("token:" + token);
        System.out.println("queryParamToken:" + queryParamToken);
        if (token != null && token.equals(queryParamToken)) {
            response.setStatus(HttpServletResponse.SC_OK);
        } else {
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
        }
    }

}
